@@ -55,7 +55,6 @@ STATUS_INVALID_EXTENSION = 1010
 STATUS_UNEXPECTED_CONDITION = 1011
 STATUS_TLS_HANDSHAKE_ERROR = 1015
 
-
 logger = logging.getLogger()
 
 class WebSocketException(Exception):
@@ -167,7 +166,7 @@ _MAX_CHAR_BYTE = (1<<8) -1
 # http://axod.blogspot.com/2010/06/websocket-gets-update-and-it-breaks.html
 
 def _create_sec_websocket_key():
-    uid = uuid.uuid1()
+    uid = uuid.uuid4()
     return base64.encodestring(uid.bytes).strip()
 
 _HEADERS_TO_CHECK = {
@@ -211,6 +210,15 @@ class ABNF(object):
     OPCODES = (OPCODE_TEXT, OPCODE_BINARY, OPCODE_CLOSE,
                 OPCODE_PING, OPCODE_PONG)
 
+    # opcode human readable string
+    OPCODE_MAP = {
+        OPCODE_TEXT: "text",
+        OPCODE_BINARY: "binary",
+        OPCODE_CLOSE: "close",
+        OPCODE_PING: "ping",
+        OPCODE_PONG: "pong"
+        }
+
     # data length threashold.
     LENGTH_7  = 0x7d
     LENGTH_16 = 1 << 16
@@ -314,14 +322,17 @@ class WebSocket(object):
     >>> ws.recv()
     'Hello, Server'
     >>> ws.close()
+    
+    get_mask_key: a callable to produce new mask keys, see the set_mask_key 
+      function's docstring for more details
     """
-    def __init__(self):
+    def __init__(self, get_mask_key = None):
         """
         Initalize WebSocket object.
         """
         self.connected = False
         self.io_sock = self.sock = socket.socket()
-        self.get_mask_key = None
+        self.get_mask_key = get_mask_key
         
     def set_mask_key(self, func):
         """
@@ -515,7 +526,11 @@ class WebSocket(object):
         """
         while True:
             frame = self.recv_frame()
-            if frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
+            if not frame:
+                # handle error: 
+                # 'NoneType' object has no attribute 'opcode'
+                raise WebSocketException("Not a valid frame %s" % frame)
+            elif frame.opcode in (ABNF.OPCODE_TEXT, ABNF.OPCODE_BINARY):
                 return (frame.opcode, frame.data)
             elif frame.opcode == ABNF.OPCODE_CLOSE:
                 self.send_close()
@@ -544,16 +559,22 @@ class WebSocket(object):
         mask = b2 >> 7 & 1
         length = b2 & 0x7f
 
+        length_data = ""
         if length == 0x7e:
-            l = self._recv(2)
-            length = struct.unpack("!H", l)[0]
+            length_data = self._recv(2)
+            length = struct.unpack("!H", length_data)[0]
         elif length == 0x7f:
-            l = self._recv(8)
-            length = struct.unpack("!Q", l)[0]
+            length_data = self._recv(8)
+            length = struct.unpack("!Q", length_data)[0]
 
+        mask_key = ""
         if mask:
             mask_key = self._recv(4)
         data = self._recv(length)
+        if traceEnabled:
+            recieved = header_bytes + length_data + mask_key + data
+            logger.debug("recv: " + repr(recieved))
+
         if mask:
             data = ABNF.mask(mask_key, data)
         
@@ -636,7 +657,7 @@ class WebSocketApp(object):
     """
     def __init__(self, url,
                  on_open = None, on_message = None, on_error = None, 
-                 on_close = None):
+                 on_close = None, keep_running = True, get_mask_key = None):
         """
         url: websocket url.
         on_open: callable object which is called at opening websocket.
@@ -651,12 +672,18 @@ class WebSocketApp(object):
          The passing 2nd arugment is exception object.
        on_close: callable object which is called when closed the connection.
          this function has one argument. The arugment is this class object.
+       keep_running: a boolean flag indicating whether the app's main loop should
+         keep running, defaults to True
+       get_mask_key: a callable to produce new mask keys, see the WebSocket.set_mask_key's
+         docstring for more information
         """
         self.url = url
         self.on_open = on_open
         self.on_message = on_message
         self.on_error = on_error
         self.on_close = on_close
+        self.keep_running = keep_running
+        self.get_mask_key = get_mask_key
         self.sock = None
 
     def send(self, data):
@@ -669,6 +696,7 @@ class WebSocketApp(object):
         """
         close websocket connection.
         """
+        self.keep_running = False
         self.sock.close()
 
     def run_forever(self):
@@ -679,10 +707,10 @@ class WebSocketApp(object):
         if self.sock:
             raise WebSocketException("socket is already opened")
         try:
-            self.sock = WebSocket()
+            self.sock = WebSocket(self.get_mask_key)
             self.sock.connect(self.url)
             self._run_with_no_err(self.on_open)
-            while True:
+            while self.keep_running:
                 data = self.sock.recv()
                 if data is None:
                     break
